<% '***************************************************************************
   '* Football Pool                                                           *
   '*                                                                         *
   '* Copyright 2000 by Mike Hall                                             *
   '* Web address: http://www.brainjar.com                                    *
   '*                                                                         *
   '* Use permitted provided this notice remains intact. Any sale or redis-   *
   '* tribution without prior consent is prohibited.                          *
   '***************************************************************************

   '***************************************************************************
   '* Constant definitions.                                                   *
   '***************************************************************************

   'Administrator username.

   Const ADMIN_USERNAME = "Admin"

   'Minimum password length

   Const PASSWORD_MIN_LEN = 6

   'Time zone difference between host server and database schedule (Eastern).

   Const TIME_ZONE_DIFF = -3

   'Cost of weekly bet.

   Const BET_AMOUNT = 5

   '***************************************************************************
   '* Global variables.                                                       *
   '***************************************************************************

   dim DbConn

   '***************************************************************************
   '* Functions and subroutines.                                              *
   '***************************************************************************

   '---------------------------------------------------------------------------
   ' OpenDB(): Opens the database connection (global variable 'DbConn').
   '---------------------------------------------------------------------------

   sub OpenDB()

     set DbConn = Server.CreateObject("ADODB.Connection")
     dbDir = Server.MapPath("./data/NFL.mdb")
     DbConn.Open "DBQ="& dbDir _
       &";Driver={Microsoft Access Driver (*.mdb)};DriverId=25;FIL=MS Access;"

   end sub

   '---------------------------------------------------------------------------
   ' CurrentDateTime(): Returns a date object representing the current date
   ' and time adjusted to the time zone used in the database.
   '---------------------------------------------------------------------------

   function CurrentDateTime()

     CurrentDateTime = DateAdd("h", -TIME_ZONE_DIFF, CDate(Date() & " " & Time()))

   end function

   '---------------------------------------------------------------------------
   ' CurrentWeek(): Returns the current week number based on the current
   ' (time zone adjusted) date.
   '---------------------------------------------------------------------------

   function CurrentWeek()

     dim sql, datenow, rs, found

     CurrentWeek = 0
     datenow = CurrentDateTime
     sql = "select Week, Date from Schedule order by Date"
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       found = false
       do while not rs.EOF and not found
         if DateDiff("d", DateValue(rs.Fields("Date").Value), datenow) <= 0 then
           found = true
           currentWeek = rs.Fields("Week").Value
         end if
         rs.MoveNext
       loop
     end if
     if CurrentWeek = 0 then
       CurrentWeek = NumberOfWeeks()
     end if

   end function

   '---------------------------------------------------------------------------
   ' NumberOfUsers(): Returns the number of users in the database excluding the
   ' administrator.
   '---------------------------------------------------------------------------

   function NumberOfUsers()

     dim sql, rs

     NumberOfUsers = 0
     sql = "select count(*) as Total from Users" _
        & " where Username <> '" & ADMIN_USERNAME & "'"
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       NumberOfUsers = rs.Fields("Total").Value
     end if

   end function

   '---------------------------------------------------------------------------
   ' NumberOfGames(week): Returns the number of games scheduled for week given.
   '---------------------------------------------------------------------------

   function NumberOfGames(week)

     dim sql, rs

     NumberOfGames = 0
     sql = "select count(*) as Total from Schedule where Week = " & week
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       NumberOfGames = rs.Fields("Total").Value
     end if

   end function

   '---------------------------------------------------------------------------
   ' NumberOfWeeks(): Returns the total number of weeks in the schedule.
   '---------------------------------------------------------------------------

   function NumberOfWeeks()

     dim sql, rs

     NumberOfWeeks = 0
     sql = "select max(Week) as Total from Schedule"
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       NumberOfWeeks = rs.Fields("Total").Value
     end if

   end function

   '---------------------------------------------------------------------------
   ' UserScore(username, week): Returns the number of correct picks for the
   ' given user in the given week. If the user did not enter any picks that
   ' week, a null string is returned.
   '---------------------------------------------------------------------------

   function UserScore(username, week)

     dim sql, rs, n

     UserScore = ""

     'Return a null string if no picks were made.

     sql = "select count(*) as total" _
        & " from Picks, Schedule" _
        & " where Username = '" & username & "'" _
        & " and Schedule.Week = " & week _
        & " and Picks.GameID = Schedule.GameID"
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       if rs.Fields("total").Value = 0 then
         exit function
       end if
     else
       exit function
     end if

     'Otherwise, return the number of correct picks.
   
     sql = "select count(*) as score" _
        & " from Picks, Schedule" _
        & " where Username = '" & username & "'" _
        & " and Schedule.Week = " & week _
        & " and Picks.GameID = Schedule.GameID" _
        & " and Picks.Pick = Schedule.Result" _
        & " and Schedule.Result <> ''"
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       UserScore = rs.Fields("score").Value
     end if

   end function

   '---------------------------------------------------------------------------
   ' UserTBGuess(username, week): Returns the given user's tie breaker
   ' guess for the given week.
   '---------------------------------------------------------------------------

   function UserTBGuess(username, week)

     dim sql, rs, n

     UserTBGuess = ""

     sql = "select TBPoints" _
        & " from TieBreaker" _
        & " where Username = '" & username & "'" _
        & " and Week = " & week
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       UserTBGuess = rs.Fields("TBPoints").Value
     end if

   end function

   '---------------------------------------------------------------------------
   ' TBPointTotal(week): Returns the actual point total of the Monday Night
   ' game for the given week. If those scores are not available, a null string
   ' is returned.
   '---------------------------------------------------------------------------

   function TBPointTotal(week)

     dim sql, rs, n

     TBPointTotal = ""
     sql = "select VisitorScore + HomeScore as total from Schedule" _
        & " where Week = " & week _
        & " order by Date desc"
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       n = rs.Fields("total").Value
       if not IsNumeric(n) then
         exit function
       end if
       TBPointTotal = n
     end if

   end function

   '---------------------------------------------------------------------------
   ' NumberOfEntries(week): Returns the total number of user's who made picks
   ' for the given week.
   '---------------------------------------------------------------------------

   function NumberOfEntries(week)

     dim sql, rs

     NumberOfEntries = 0
     sql = "select distinct Username" _
         & " from Picks, Schedule" _
         & " where Week = " & week _
         & " and Picks.GameID = Schedule.GameID"
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       do while not rs.EOF
         NumberOfEntries = NumberOfEntries + 1
         rs.MoveNext
       loop
     end if

   end function

   '---------------------------------------------------------------------------
   ' WinnersList(week): Compares scores and tie breakers to determine the
   ' winner or winners for the given week. An array of the winners user names
   ' is returned. If the week has not been concluded, or no players entered
   ' that week, a null string is returned instead.
   '---------------------------------------------------------------------------

   function WinnersList(week)

     dim sql, rs
     dim tbActual
     dim highScore, lowDiff
     dim username, score, tb, diff
     dim list

     WinnersList = ""

     'Get point total for the tie breaker, exit if not available.

     tbActual = TBPointTotal(week)
     if not IsNumeric(tbActual) then
       exit function
     end if

     'Initialize current high score and tie breaker difference.

     highScore = -1
     lowDiff   =  0

     'Check all users.

     sql = "select Username" _
        & " from Users" _
        & " where Username <> '" & ADMIN_USERNAME & "'"
     set rs = DbConn.Execute(sql)
     if not (rs.BOF and rs.EOF) then
       do while not rs.EOF
         username = rs.Fields("Username").Value
         score = UserScore(username, week)
         tb = UserTBGuess(username, week)

         if IsNumeric(score) and IsNumeric(tb) then

           'Compare user's score to the current highest.

           diff = Abs(tbActual - tb)
           if score > highScore or (score = highScore and diff < lowDiff) then
             redim list(0)
             list(0) = username
             highScore = score
             lowDiff = diff
           elseif score = highScore and diff = lowDiff then
             redim preserve list(UBound(list) + 1)
             list(UBound(list)) = username
           end if           
         end if

         rs.MoveNext
       loop
     end if

     WinnersList = list

   end function

   '---------------------------------------------------------------------------
   ' OpenDates(week): Returns a string of names for teams that do not have a
   ' game scheduled for given week.
   '---------------------------------------------------------------------------

   function OpenDates(week)

     dim sql, rs, rs2
     dim id, team

     OpenDates = ""

     sql = "select * from Teams" _
        & " order by City, Name"
     set rs = DbConn.Execute(sql)
     do while not rs.EOF
       id = rs.Fields("TeamID").Value
       sql = "select count(*) as total" _
          & " from Schedule" _
          & " where (HomeID = '" & id & "'" _
          & " or VisitorID = '" & id & "')" _
          & " and week = " & week
       set rs2 = DbConn.Execute(sql)
       if rs2.Fields("total").Value = 0 then
         team = rs.Fields("City").Value
         if rs.Fields("DisplayName").Value <> "" then
           team = rs.Fields("DisplayName").Value
         end if
         if OpenDates <> "" then
           OpenDates = OpenDates & ", "
         end if
         OpenDates = OpenDates & team
       end if
       rs.MoveNext
     loop

   end function

   '---------------------------------------------------------------------------
   ' FormatTime(dt): Returns a formatted string based on the time value of the
   ' given date.
   '---------------------------------------------------------------------------

   function FormatTime(dt)

     dim hh, mm, symbol

     hh = Hour(dt)
     mm = Minute(dt)
     if mm < 10 then
     mm = "0" & mm
     end if
     if hh >= 12 then
       symbol = "pm"
     else
       symbol = "am"
     end if
     if hh >= 13 then
       hh = hh - 12
     end if
     FormatTime = hh & ":" & mm & " " & symbol

   end function

   '---------------------------------------------------------------------------
   ' FormatDate(dt): Returns a formatted string based on the date value of the
   ' given date.
   '---------------------------------------------------------------------------

   function FormatDate(dt)

     FormatDate = Month(dt) & "/" & Day(dt)

   end function

   '---------------------------------------------------------------------------
   ' FormatAmount(n): Returns a formatted string based on the given number as
   ' a dollar amount.
   '---------------------------------------------------------------------------

   function FormatAmount(n)

     FormatAmount = "&nbsp;"
     if IsNumeric(n) then
       FormatAmount = FormatCurrency(n, 2, true, false, true)
     end if

   end function

   '---------------------------------------------------------------------------
   ' FormatPercent(n): Returns a formatted string based on the given number as
   ' a percentage.
   '---------------------------------------------------------------------------

   function FormatPercent(n)

     FormatPercent = "&nbsp;"
     if IsNumeric(n) then
       FormatPercent = FormatNumber(Round(n, 3), 3, true)
     end if

   end function

   '---------------------------------------------------------------------------
   ' CreateSalt(): Returns an eight-character random salt string. Used for
   ' generating the hash for passwords.
   '---------------------------------------------------------------------------

   function CreateSalt()

     dim i, n

     CreateSalt = ""
     Randomize
     for i = 1 to 8
       n = Int(26 * Rnd()) + Asc("a")
       CreateSalt = CreateSalt & Chr(n)
     next

   end function

   '---------------------------------------------------------------------------
   ' Hash(str): Calculates a 16-character hash value for the given string. Used
   ' for storing passwords.
   '---------------------------------------------------------------------------

   function Hash(ByVal str)

     dim i, j, k, l, m, n, tmp
     dim h(7), c(63)

     Hash = ""

     h(0) = &H10
     h(1) = &HFE
     h(2) = &H32
     h(3) = &HDC
     h(4) = &H54
     h(5) = &HBA
     h(6) = &H76
     h(7) = &H98

     str = LCase(str)
     l = Len(str)
     for i = 0 to l - 1
       c(i) = Asc(Mid(str, i + 1, 1))
     next
     for i = l to UBound(c) - 1
       c(i) = 0
     next
     c(UBound(c)) = l

     l = UBound(c)
     m = 0
     n = 0
     for i = 0 to l
       j = (i + 2) mod l
       m = (m + c(i) xor c(j)) mod 256
       n = (n + c(i) or  c(j))  mod 256
       j = i mod (UBound(h) + 1)
       h(j) = (h(j) + (m xor n)) mod 256
     next

     for i = 0 to UBound(h)
       tmp = Hex(h(i))
       if Len(tmp) < 2 then
         tmp = "0" + tmp
       end if
       Hash = Hash & tmp
     next

   end function

   '---------------------------------------------------------------------------
   ' ErrorMessage(msg): Outputs 'msg' in the error style.
   '---------------------------------------------------------------------------

   sub ErrorMessage(msg)

     Response.Write("<span class=""error"">" & msg & "</span><br>")

   end sub

   '---------------------------------------------------------------------------
   ' InfoMessage(msg): Outputs 'msg' in the info style.
   '---------------------------------------------------------------------------

   sub InfoMessage(msg)

     Response.Write("<span class=""info"">" & msg & "</span><br>")

   end sub %>
